/*-
 * Copyright (c) 1999, 2000, 2001, 2002, 2003 Lev Walkin <vlm@lionet.info>.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: htmlquote.c,v 1.4 2005/05/25 21:55:10 vlm Exp $
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <errno.h>

#ifdef	HAVE_CONFIG_H
#include "config.h"
#endif

#include <sf_cgi.h>
#include <sf_mem.h>
#include <sf_smisc.h>

/*
 * html_quote storage.
 */
static char *_sf_hq_buf = NULL;

char *
html_quote_detach() {
	char *buf = _sf_hq_buf;
	_sf_hq_buf = NULL;
	return buf;
}

char *
html_quote(const char *s) {
	char *buf;
	char *p;
	int n;
	int nc;
	char sc;

	if(s == NULL)
		s = "";

	nc = countchar2(s, "\"'&<>");
	n = strlen(s) + sizeof("&quot;") * nc + 1;

	buf = (char *)sf_malloc(n);
	if(buf == NULL)
		/* ENOMEM? */
		return NULL;

	if(nc == 0) {
		memcpy(buf, s, n);

		if(_sf_hq_buf)
			free(_sf_hq_buf);

		return (_sf_hq_buf = buf);
	}

	for(p = buf; (sc=*s); s++) {
		switch(sc) {
			case '\'':
				strcpy(p, "&#39;"); p+=5; break;
			case '"':
				strcpy(p, "&quot;"); p+=6; break;
			case '<':
				strcpy(p, "&lt;"); p+=4; break;
			case '>':
				strcpy(p, "&gt;"); p+=4; break;
			case '&':
				strcpy(p, "&amp;"); p+=5; break;
			default:
				*p++ = sc; continue;
			}
	}
	*p = 0;

	if(_sf_hq_buf)
		free(_sf_hq_buf);

	return (_sf_hq_buf = buf);
}

