/*-
 * Copyright (c) 1999, 2000, 2001, 2002, 2003 Lev Walkin <vlm@lionet.info>.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: langprefs.c,v 1.2 2005/05/25 21:55:10 vlm Exp $
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>

#include <sf_cgi.h>


svect *
getlanguageprefs() {
	static svect *sl = NULL;
	char *p;
	int i;
	register char *t;

	if(sl)	/* Already parsed */
		return sl;

	p = getenv("HTTP_ACCEPT_LANGUAGE");
	if(p == NULL) {
		errno = ESRCH;
		return NULL;
	}

	sl = split(p, ", ", 0);
	if(sl == NULL)
		/* ENOMEM? */
		return NULL;

	for(i = 0; i < sl->count; i++) {
		p = strchr(sl->list[i], ';');
		if(p) {
			*p = '\0';
			if((p - sl->list[i]) == 0) {
				sdel(sl, i--);
				continue;
			}
		}

		for(t = sl->list[i]; *t; t++) {
			if(*t == '-')
				continue;
			if(*t >= 'a' && *t <= 'z')
				continue;
			if(*t >= '0' && *t <= '9')
				continue;
			if(*t >= 'A' && *t <= 'Z')
				continue;
			break;
		}
		if(*t)	/* Not reached the end */
			sdel(sl, i--);
	}

	if(sl->count == 0) {
		sfree(sl);
		sl = NULL;
	}

	return sl;
}


