/*-
 * Copyright (c) 1999, 2000, 2001, 2002 Lev Walkin <vlm@lionet.info>.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: eq.c,v 1.2 2005/05/25 21:55:10 vlm Exp $
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <sys/types.h>
#include <errno.h>

#ifdef	HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef	HAVE_UNISTD_H
#include <unistd.h>
#endif

#include <sf_smisc.h>


/*
 * Equality test. Case-sensitive.
 */
int
eq(const char *a, const char *b) {

	/* Can't test equality of NOTHING */
	if(a == NULL || b == NULL)
		return 0;

	return strcmp(a, b)?0:1;
}

/*
 * Equality test. Case-insensitive.
 */
int
ecq(const char *a, const char *b) {
	register const unsigned char
		*us1 = (const unsigned char *)a,
		*us2 = (const unsigned char *)b;

	/* Can't test equality of NOTHING */
	if(us1 == NULL || us2 == NULL)
		return 0;

	while (tolower(*us1) == tolower(*us2++))
		if (*us1++ == '\0')
			return 1;

	return 0;
}

/*
 * Equality test. Case-insensitive. b should be already lower-cased.
 */
int
ecq2(const char *a, const char *b) {
	register const unsigned char
		*us1 = (const unsigned char *)a,
		*us2 = (const unsigned char *)b;

	/* Can't test equality of NOTHING */
	if(us1 == NULL || us2 == NULL)
		return 0;

	while (tolower(*us1) == *us2++)
		if (*us1++ == '\0')
			return (1);

	return 0;
}
